package me.km.blocks;

import me.km.fluids.BlockFluidBase;
import me.km.fluids.BlockFluidHoney;
import me.km.fluids.BlockFluidPoison;
import me.km.fluids.ModFluids;
import me.km.items.ItemMetal;
import net.minecraft.block.Block;
import net.minecraft.block.material.MapColor;
import net.minecraft.block.material.Material;
import net.minecraft.init.Blocks;
import net.minecraft.item.ItemBlock;
import net.minecraftforge.fml.common.registry.ExistingSubstitutionException;
import net.minecraftforge.fml.common.registry.GameRegistry;

public class ModBlocks 
{
    // ores + blocks
    public static BlockOre copperOre;
    public static BlockBase copperBlock;
    
    public static BlockOre tinOre;
    public static BlockBase tinBlock;
    
    public static BlockBase bronzeBlock;
    
    public static BlockOre silverOre;
    public static BlockBase silverBlock;
    
    // misc
    public static BlockBase guildblock;
    public static BlockBase artefact;
    public static BlockHay realHayBlock;
    public static BlockHayBed realHayBed;
    
    // traps
    public static BlockSpikeTrap spikes;
    
    // fluids
    public static BlockFluidPoison poison;
    public static BlockFluidHoney honey;
    
    public static void init() 
    {
        // ores + blocks
        copperOre = register(new BlockOre("copper_ore", "oreCopper"));      
        copperBlock = register((BlockBase) new BlockBase(Material.IRON, "copper_block", "blockCopper")
                .setMapColor(MapColor.BROWN).setHardness(3).setResistance(10));
        tinOre = register(new BlockOre("tin_ore", "oreTin"));      
        tinBlock = register((BlockBase) new BlockBase(Material.IRON, "tin_block", "blockTin")
                .setMapColor(MapColor.IRON).setHardness(3).setResistance(5));
        bronzeBlock = register((BlockBase) new BlockBase(Material.IRON, "bronze_block", "blockBronze")
                .setMapColor(MapColor.YELLOW).setHardness(5).setResistance(10));
        silverOre = register(new BlockOre("silver_ore", "oreSilver"));      
        silverBlock = register((BlockBase) new BlockBase(Material.IRON, "silver_block", "blockSilver")
                .setMapColor(MapColor.IRON).setHardness(5).setResistance(10));
        
        // misc
        guildblock = register((BlockBase) new BlockBase(Material.IRON, "guild_block", "blockGuild")
                .setHardness(50.0F).setResistance(2000.0F));
        artefact = register((BlockBase) new BlockBase(Material.IRON, "artefact", "artefact")
                .setHardness(50.0F).setResistance(2000.0F));
        realHayBlock = register(new BlockHay("real_hay_block", "realHayBlock"));
        realHayBed = registerBlock(new BlockHayBed("real_hay_bed", "realHayBlock"));
        
        // traps
        spikes = registerMetal(new BlockSpikeTrap(Material.IRON, "spikes", "spikes"));
    
        // fluids
        poison = register(new BlockFluidPoison(ModFluids.poison, Material.WATER));
        honey = register(new BlockFluidHoney(ModFluids.honey, Material.WATER));
    }
    
    // THIS IS ONLY FOR REPLACING VANILLA BLOCKS
    public static void lateInit() 
    {
        BlockTallGrass grass = new BlockTallGrass();
        overwriteBlock(Blocks.TALLGRASS, grass);
    }
    
    @SuppressWarnings("")
    private static void overwriteBlock(Block old, Block block)
    {
        try
        {
            GameRegistry.addSubstitutionAlias(old.getRegistryName().toString(), 
                    GameRegistry.Type.BLOCK, 
                    block.setRegistryName(old.getRegistryName().getResourcePath()).setUnlocalizedName(old.getUnlocalizedName().substring(5)));
        }
        catch(ExistingSubstitutionException | NullPointerException ex)
        {
            System.out.println(ex);
        }
    }
    //END OF REPLACING VANILLA BLOCKS

    private static <T extends Block> T register(T block, ItemBlock itemBlock) 
    {
        GameRegistry.register(block);
        GameRegistry.register(itemBlock);
        if(block instanceof IBlockBase) 
        {
            ((IBlockBase) block).registerItemModel(itemBlock);
        }
        if(block instanceof BlockFluidBase) 
        {
            ((BlockFluidBase) block).registerBlockModel();
        }
        return block;
    }
    
    private static <T extends Block> T registerBlock(T block) 
    {
        GameRegistry.register(block);
        return block;
    }

    private static <T extends Block> T register(T block) 
    {
        ItemBlock itemBlock = new ItemBlock(block);
        itemBlock.setRegistryName(block.getRegistryName());
        return register(block, itemBlock);
    }
    
    private static <T extends Block> T registerMetal(T block) 
    {
        ItemMetal itemBlock = new ItemMetal(block);
        itemBlock.setRegistryName(block.getRegistryName());
        return register(block, itemBlock);
    }
}
