package me.km.dimensions;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import me.km.KajetansMod;
import me.km.api.GlobalText;
import me.km.api.Location;
import me.km.api.Module;
import me.km.api.ModuleCommand;
import me.km.api.Utils;
import me.km.exception.PlayerNotFoundException;
import me.km.permissions.Permissions;
import net.minecraft.command.ICommandSender;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.EnumDifficulty;
import net.minecraft.world.WorldServer;

public class CommandWorld extends ModuleCommand
{
    public CommandWorld(Module m) 
    {
        super("world", m);
        super.setDescription("Befehle für die Verwaltung der Welten");
        super.setUsage("/world für die Hilfe");
        super.setPermission(Permissions.WORLD);
    }

    private boolean doesWorldExist(String name)
    {
        return new File("./" + name).isDirectory();       
    }  
    
    private ArrayList<String> getAllWorlds()
    {
        File worlds = new File("./"); 
        ArrayList<String> list = new ArrayList<>();
        for(File f : worlds.listFiles())
        {
            if(!f.isDirectory())
            {
                continue;
            }
            if(new File(f, "level.dat").exists())
            {
                list.add(f.getName());
            }
        }
        return list;       
    }
    
    @Override
    public boolean execute(ICommandSender cs, String[] arg) 
    {
        Module m = this.getModule();
        if(arg.length >= 1)
        {
            switch(arg[0].toLowerCase())
            {
                case "tp":
                {
                    if(arg.length >= 2)
                    {
                        WorldServer w = ModDimensions.getWorldFromName(arg[1]);
                        if(w == null)
                        {
                            m.send(cs, "Die Welt '" + arg[1] + "' wurde nicht gefunden.");
                            return true;
                        }
                        EntityPlayerMP affectedEntityPlayerMP;
                        try
                        {
                            affectedEntityPlayerMP = Utils.getPlayerByName(arg[2]);
                        }
                        catch(PlayerNotFoundException ex)
                        {
                            m.send(cs, GlobalText.cantFindPlayer(arg[2]));
                            return true;
                        }
                        catch(IndexOutOfBoundsException ex)
                        {
                            if(!(cs instanceof EntityPlayerMP))
                            {
                                m.send(cs, GlobalText.missingParameter());
                                return true;
                            }
                            affectedEntityPlayerMP = (EntityPlayerMP) cs;
                        }
                        BlockPos pos = w.getSpawnPoint();
                        while(pos.getY() <= 255 && !w.isAirBlock(pos))
                        {
                            pos = pos.add(0, 1, 0);
                        }
                        Utils.teleportEntity(affectedEntityPlayerMP, new Location(w, pos));
                        return true;
                    }
                    break;
                }
                case "setspawn":
                {
                    if(!(cs instanceof EntityPlayerMP))
                    {
                        m.send(cs, GlobalText.onlyPlayer());
                        return true;
                    }
                    EntityPlayerMP p = (EntityPlayerMP) cs;
                    p.getEntityWorld().setSpawnPoint(p.getPosition());
                    m.send(cs, "Der Spawn der Welt '" + p.getEntityWorld().getWorldInfo().getWorldName() + "' wurde gesetzt.");
                    return true;
                }
                case "list":
                {
                    m.send(cs, "Folgende Welten sind geladen:");
                    Arrays.stream(KajetansMod.server.worlds).forEach((w) -> 
                    {
                        m.sendListElement(cs, ModDimensions.getWorldName(w));
                    });
                    return true;
                }
                case "listall":
                {
                    m.send(cs, "Folgende Welten sind verfügbar:");
                    getAllWorlds().stream().forEach((s) -> 
                    {
                        m.sendListElement(cs, s);
                    });
                    return true;
                }
                case "load":
                {
                    if(arg.length >= 2)
                    {                       
                        if(!doesWorldExist(arg[1]))
                        {
                            m.send(cs, "Die Welt '" + arg[1] + "' wurde nicht gefunden.");
                            return true;
                        }
                        if(ModDimensions.loadWorld(arg[1], ModDimensions.CUSTOM_WORLD))
                        {
                            m.send(cs, "'" + arg[1] + "' wurde geladen.");
                        }
                        else
                        {
                            m.send(cs, "'" + arg[1] + "' ist bereits geladen.");
                        }
                        return true;
                    }
                    break;
                }
                case "unload":
                {
                    if(arg.length >= 2)
                    {                       
                        if(ModDimensions.unloadWorld(arg[1]))
                        {
                            m.send(cs, "'" + arg[1] + "' wurde aus dem Speicher entfernt.");
                            return true;
                        }
                        m.send(cs, "'" + arg[1] + "' kann nicht entfernt werden.");
                        return true;
                    }
                    break;
                }
                case "diffi":
                {
                    if(arg[0].equals("diffi") && arg.length >= 3)
                    {
                        WorldServer w = ModDimensions.getWorldFromName(arg[1]);
                        if(w == null)
                        {
                            m.send(cs, "Die Welt '" + arg[1] + "' wurde nicht gefunden.");
                            return true;
                        }
                        try
                        {
                            EnumDifficulty diffi = EnumDifficulty.valueOf(arg[2].toUpperCase());
                            w.getWorldInfo().setDifficulty(diffi);
                            m.send(cs, "Schwierigkeit in der Welt '" + arg[1] + "' ist nun '" + Utils.formatString(diffi.name()) + "'.");
                            return true;
                        }
                        catch(IllegalArgumentException ex)
                        {
                            m.send(cs, "Diese Schwierigkeitsstufe gibt es nicht.");
                            return true;
                        }                                         
                    }
                    break;
                }
                case "info":
                {
                    if(arg.length >= 2)
                    {
                        WorldServer w = ModDimensions.getWorldFromName(arg[1]);
                        if(w == null)
                        {
                            m.send(cs, "Die Welt '" + arg[1] + "' wurde nicht gefunden.");
                            return true;
                        }                          
                        m.send(cs, "Infos zur Welt '" + arg[1] + "'");
                        for(String rule : w.getGameRules().getRules())
                        {
                            m.sendListElement(cs, rule + " = " + w.getGameRules().getString(rule));
                        }                     
                        m.sendListElement(cs, "Zeit = " + w.getWorldInfo().getWorldTime());
                        m.sendListElement(cs, "Schwierigkeit = " + Utils.formatString(w.getDifficulty().name()));  
                        
                        WorldPreferences wp = KajetansMod.worldManager.getWorldPreferences(w);
                        
                        m.sendListElement(cs, "Block-Protection = " + wp.blockProtection);    
                        m.sendListElement(cs, "Default-Enchanting = " + wp.defaultEnchanting);    
                        m.sendListElement(cs, "PVP-Protection = " + wp.pvpProtection);    
                        m.sendListElement(cs, "Skills = " + wp.skills);   
                        m.sendListElement(cs, "Statuseffekte = " + wp.statusEffects);  
                        m.sendListElement(cs, "Schriftrollen = " + wp.scrolls);  
                        m.sendListElement(cs, "Standard-Craften = " + wp.defaultProducing);  
                        m.sendListElement(cs, "Manabar = " + wp.manabar);  
                        return true;
                    }
                    break;
                }
            }
        }            
        
        // Help Menu
        m.send(cs, "/world ...");
        m.sendHelpListElement(cs, "info <world>", "Gibt Infos über die Welt");
        m.sendHelpListElement(cs, "tp <id> [player]", "Bringt den Spieler in eine andere Welt");
        m.sendHelpListElement(cs, "list", "Zeigt alle geladenen Welten");
        m.sendHelpListElement(cs, "listall", "Zeigt alle Welten");
        m.sendHelpListElement(cs, "load <world>", "Lädt eine Welt");
        m.sendHelpListElement(cs, "unload <world>", "Entfernt die Welt aus dem Speicher");
        m.sendHelpListElement(cs, "setspawn", "Setzt den Welten-Spawn");
        m.sendHelpListElement(cs, "diffi <world> <diffi>", "Setzt die Schwierigkeit");
        return true;
    }
}
