package me.km.snuviscript;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import me.km.KajetansMod;
import me.km.api.GlobalText;
import me.km.api.Module;
import me.km.api.ModuleCommand;
import me.km.api.Utils;
import me.km.chatmanager.ChatManager;
import me.km.exception.PlayerNotFoundException;
import me.km.permissions.Permissions;
import net.minecraft.command.ICommandSender;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.util.text.TextComponentString;

public class CommandQuest extends ModuleCommand
{
    public CommandQuest(Module m) 
    {
        super("quest", m);
        super.setDescription("Zeigt alles über Quests");
        super.setUsage("/quest für die Hilfe");
        super.setPermission(Permissions.QUEST);
        super.addAlias("q");
    }

    @SuppressWarnings("unchecked")
    @Override
    public boolean execute(ICommandSender cs, String[] arg) 
    {
        Module m = this.getModule();
        if(arg.length >= 1)
        {
            final EntityPlayer p;
            if(cs instanceof EntityPlayer)
            {
                p = (EntityPlayer) cs;
            }
            else
            {
                p = null;
            }
            
            // Abkürzungen
            switch(arg[0].toLowerCase())
            {
                case "s":
                case "start":
                {
                    if(arg.length >= 3)
                    {
                        EntityPlayer affectedPlayer;
                        try
                        {
                            affectedPlayer = Utils.getPlayerByName(arg[1]);                   
                        }
                        catch(PlayerNotFoundException ex)
                        {
                            m.send(cs, GlobalText.cantFindPlayer(arg[1]));
                            return true;
                        }                           
                        KajetansMod.scripts.startQuest(cs, arg[Utils.randomInt(2, arg.length-1)], affectedPlayer, null, null);
                        return true;
                    }
                    break;
                }
                case "v":
                case "variable":    
                {
                    if(arg.length >= 2 && p != null)
                    {
                        Script qd = KajetansMod.scripts.getQuestData(p);
                        if(qd == null)
                        {
                            m.send(cs, "Du hast gerade keine Quest.");
                            return true;
                        }
                        m.send(cs, arg[1] + " = " + qd.getVar(arg[1]));
                        return true; 
                    }
                    break;
                }
                case "n":
                case "next":
                {
                    if(p != null)
                    {
                        Script qd = KajetansMod.scripts.getQuestData(p);
                        if(qd == null)
                        {
                            m.send(cs, "Du hast gerade keine Quest.");
                            return true;
                        }
                        qd.setIsWaiting(false);
                        qd.runCode();
                        return true;
                    }
                    break;
                }
                case "c":
                case "code":
                {
                    if(arg.length >= 2 && p != null)
                    {
                        Script qd = KajetansMod.scripts.getQuestData(p);
                        if(qd == null)
                        {
                            m.send(cs, "Du hast gerade keine Quest.");
                            return true;
                        }
                        qd.runCodeLine(ChatManager.colorMessage(Utils.connectSpaces(arg, 1), p), cs);
                        return true;
                    }
                    break;
                }
                case "p":
                case "printvars":
                {
                    if(p != null)
                    {
                        Script qd = KajetansMod.scripts.getQuestData(p);
                        if(qd == null)
                        {
                            m.send(cs, "Du hast gerade keine Quest.");
                            return true;
                        }
                        cs.sendMessage(new TextComponentString(GlobalText.Spacer()));
                        qd.getVars().entrySet().stream().filter(e -> arg.length < 2 || 
                                (e.getKey().startsWith(arg[1]))).forEach(e -> 
                        {
                            m.send(cs, e.getKey() + " = " + e.getValue());
                        });                   
                        return true; 
                    }
                    break;
                }
                case "pl":
                case "printlists":
                {
                    if(p != null)
                    {
                        Script qd = KajetansMod.scripts.getQuestData(p);
                        if(qd == null)
                        {
                            m.send(cs, "Du hast gerade keine Quest.");
                            return true;
                        }
                        cs.sendMessage(new TextComponentString(GlobalText.Spacer()));
                        qd.getVars().entrySet().stream()
                                .filter(e -> e.getValue() instanceof List)
                                .filter(e -> arg.length < 2 || (e.getKey().startsWith(arg[1])))
                                .forEach(e -> 
                        {
                            m.send(cs, e.getKey());
                            ((List<Object>) e.getValue()).forEach(l -> m.sendListElement(cs, l.toString()));
                        });                   
                        return true; 
                    }
                    break;
                }
                case "pm":
                case "printmaps":
                {
                    if(p != null)
                    {
                        Script qd = KajetansMod.scripts.getQuestData(p);
                        if(qd == null)
                        {
                            m.send(cs, "Du hast gerade keine Quest.");
                            return true;
                        }
                        cs.sendMessage(new TextComponentString(GlobalText.Spacer()));
                        qd.getVars().entrySet().stream()
                                .filter(e -> e.getValue() instanceof Map)
                                .filter(e -> arg.length < 2 || (e.getKey().startsWith(arg[1]))).forEach(e -> 
                        {
                            m.send(cs, e.getKey());
                            ((Map) e.getValue()).forEach((k, v) -> m.sendListElement(cs, k + " | " + v));
                        });                   
                        return true; 
                    }
                    break;
                }
                case "see":
                {
                    Collection<Script> quests = KajetansMod.scripts.getQuests();
                    if(quests.isEmpty())
                    {
                        m.send(cs, "Momentan sind keine Quests aktiv.");
                        return true;
                    }
                    m.send(cs, "Folgende Quests sind aktiv:");
                    quests.forEach(data ->
                    {
                        m.sendHelpListElement(cs, data.getName() + " (" + data.getId() + ")", String.join(", ", data.getPlayerNames()));
                    });
                    return true;
                }
                case "get":
                {
                    if(p != null && arg.length >= 2)
                    {
                        EntityPlayer affectedPlayer;
                        try
                        {
                            affectedPlayer = Utils.getPlayerByName(arg[1]);                   
                        }
                        catch(PlayerNotFoundException ex)
                        {
                            m.send(cs, GlobalText.cantFindPlayer(arg[1]));
                            return true;
                        }
                        KajetansMod.scripts.addPlayerToPlayer(p, affectedPlayer); 
                        return true;
                    }
                    break;
                }
                case "term":
                {
                    KajetansMod.scripts.removeQuests();
                    m.send(cs, "Alle aktiven Quests wurden beendet.");
                    return true;
                }
                case "stack":
                {
                    KajetansMod.scripts.getQuestParser().printStack = !KajetansMod.scripts.getQuestParser().printStack;
                    if(KajetansMod.scripts.getQuestParser().printStack)
                    {
                        m.send(cs, "Exceptions werden nun angezeigt.");
                        return true;
                    }
                    m.send(cs, "Exceptions werden nicht mehr angezeigt.");
                    return true;
                }
            }
        }
        
        m.send(cs, "/quest ...");
        m.sendHelpListElement(cs, "start <pname> <qn1> [qn2] ...", "Startet eine Quest");
        m.sendHelpListElement(cs, "variable <name>", "Zeigt den Wert einer Variable");
        m.sendHelpListElement(cs, "printvars [starts]", "Zeigt alle Variablen");
        m.sendHelpListElement(cs, "printlists [starts]", "Zeigt alle Listen");
        m.sendHelpListElement(cs, "printmaps [starts]", "Zeigt alle Maps");
        m.sendHelpListElement(cs, "next", "Springt in die nächste Codezeile");
        m.sendHelpListElement(cs, "code <code>", "Führt den angegebenen Code aus");
        m.sendHelpListElement(cs, "see", "Gibt alle aktiven Quests aus");
        m.sendHelpListElement(cs, "get <pname>", "Fügt dich der Quest hinzu");
        m.sendHelpListElement(cs, "term", "Entfernt alle Quests aus dem RAM");
        m.sendHelpListElement(cs, "stack", "Zeigt Exceptions auf der Konsole");
        return true;
    }
}

