package me.km.table;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Map;

public class Table 
{
    private int sortColumn;
    private final int columns;
    private final ArrayList<Object[]> list;
    
    public Table(int columns)
    {
        this.columns = columns;
        sortColumn = 0;      
        list = new ArrayList<>();
    }
    
    public Table(Map<Object, Object> map)
    {
        this.columns = 2;
        sortColumn = 0;
        list = new ArrayList<>();
        map.forEach((k, v) -> 
        {
            list.add(new Object[] {k, v});
        });
    }
    
    public ArrayList<Object[]> getRows()
    {
        return list;
    }

    public int getColumns() 
    {
        return columns;
    }

    public void addRow(Object[] o)
    {
        list.add(o);
    }
    
    public void removeRow(int i)
    {
        list.remove(i);
    }
    
    public Object getElement(int row, int column)
    {
        return list.get(row)[column];
    }
    
    public void setSortColumn(int column)
    {
        if(column >= columns)
        {
            throw new ArrayIndexOutOfBoundsException(column);
        }
        sortColumn = column;   
    }
    
    public int getIndexOf(Object o)
    {
        int i = 0;
        for(Object[] ob : list)
        {
            if(ob[sortColumn].equals(o))
            {
                return i;
            }
            i++;
        }
        return -1;
    }
    
    @SuppressWarnings("unchecked")
    public void sort()
    {
        list.sort((Object o1, Object o2) -> 
        {
            if(o1 instanceof Object[] && o2 instanceof Object[]) 
            {
                return ((Comparable) ((Object[]) o1)[sortColumn]).compareTo(((Object[]) o2)[sortColumn]);
            }
            return 0;
        });
    }
    
    public void reverse()
    {
        Collections.reverse(list);
    }
    
    public void shuffle()
    {
        Collections.shuffle(list);
    }
    
    public int getSize()
    {
        return list.size();
    }
    
    public void clear()
    {
        list.clear();
    }
}
